package sun.security.krb5.internal.rcache;

import java.util.Set;
import java.nio.file.LinkOption;
import java.nio.file.StandardCopyOption;
import java.nio.file.CopyOption;
import java.nio.BufferUnderflowException;
import java.nio.file.attribute.FileAttribute;
import java.nio.ByteOrder;
import java.nio.ByteBuffer;
import java.nio.file.StandardOpenOption;
import java.nio.file.OpenOption;
import java.nio.file.Files;
import java.nio.file.attribute.PosixFilePermission;
import java.util.HashSet;
import java.nio.channels.SeekableByteChannel;
import java.io.Closeable;
import java.io.IOException;
import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.KerberosTime;
import java.io.File;
import java.nio.file.Path;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.security.krb5.internal.ReplayCache;

public class DflCache extends ReplayCache
{
    private static final int KRB5_RV_VNO = 1281;
    private static final int EXCESSREPS = 30;
    private final String source;
    private static int uid;
    
    public DflCache(String source) {
        this.source = source;
    }
    
    private static String defaultPath() {
        return (String)AccessController.doPrivileged(new GetPropertyAction("java.io.tmpdir"));
    }
    
    private static String defaultFile(String str) {
        int endIndex = str.indexOf(47);
        if (endIndex == -1) {
            endIndex = str.indexOf(64);
        }
        if (endIndex != -1) {
            str = str.substring(0, endIndex);
        }
        if (DflCache.uid != -1) {
            str = str + "_" + DflCache.uid;
            return str;
        }
        return str;
    }
    
    private static Path getFileName(String substring, String s) {
        String parent;
        String child;
        if (substring.equals("dfl")) {
            parent = defaultPath();
            child = defaultFile(s);
        }
        else if (substring.startsWith("dfl:")) {
            substring = substring.substring(4);
            int lastIndex = substring.lastIndexOf(47);
            int lastIndex2 = substring.lastIndexOf(92);
            if (lastIndex2 > lastIndex) {
                lastIndex = lastIndex2;
            }
            if (lastIndex == -1) {
                parent = defaultPath();
                child = substring;
            }
            else if (new File(substring).isDirectory()) {
                parent = substring;
                child = defaultFile(s);
            }
            else {
                parent = null;
                child = substring;
            }
        }
        else {
            throw new IllegalArgumentException();
        }
        return new File(parent, child).toPath();
    }
    
    @Override
    public void checkAndStore(KerberosTime kerberosTime, AuthTimeWithHash authTimeWithHash) throws KrbApErrException {
        try {
            this.checkAndStore0(kerberosTime, authTimeWithHash);
        }
        catch (IOException cause) {
            KrbApErrException ex = new KrbApErrException(60);
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private synchronized void checkAndStore0(KerberosTime kerberosTime, AuthTimeWithHash authTimeWithHash) throws IOException, KrbApErrException {
        Path fileName = getFileName(this.source, authTimeWithHash.server);
        int n = 0;
        try (final DflCache.Storage storage = new DflCache.Storage()) {
            try {
                n = DflCache.Storage.access$100(storage, fileName, authTimeWithHash, kerberosTime);
            }
            catch (IOException ex) {
                DflCache.Storage.access$200(fileName);
                n = DflCache.Storage.access$100(storage, fileName, authTimeWithHash, kerberosTime);
            }
            DflCache.Storage.access$300(storage, authTimeWithHash);
        }
        if (n > 30) {
            DflCache.Storage.access$400(fileName, kerberosTime);
        }
    }
    
    static {
        try {
            Class<Object> forName = Class.forName("com.sun.security.auth.module.UnixSystem");
            DflCache.uid = (int)(long)forName.getMethod("getUid", new Class[0]).invoke(forName.newInstance(), new Object[0]);
        }
        catch (Exception ex) {
            DflCache.uid = -1;
        }
    }
    
    private static class Storage implements Closeable
    {
        SeekableByteChannel chan;
        
        private static void create(Path path) throws IOException {
            SeekableByteChannel noClose = createNoClose(path);
            __Null _Null = null;
            if (noClose != null) {
                if (_Null != null) {
                    try {
                        noClose.close();
                    }
                    catch (Throwable exception) {
                        _Null.addSuppressed(exception);
                    }
                }
                else {
                    noClose.close();
                }
            }
            makeMine(path);
        }
        
        private static void makeMine(Path path) throws IOException {
            try {
                HashSet<PosixFilePermission> perms = new HashSet();
                perms.add(PosixFilePermission.OWNER_READ);
                perms.add(PosixFilePermission.OWNER_WRITE);
                Files.setPosixFilePermissions(path, perms);
            }
            catch (UnsupportedOperationException ex) {}
        }
        
        private static SeekableByteChannel createNoClose(Path path) throws IOException {
            SeekableByteChannel byteChannel = Files.newByteChannel(path, new OpenOption[] { StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING, StandardOpenOption.WRITE });
            ByteBuffer allocate = ByteBuffer.allocate(6);
            allocate.putShort((short)1281);
            allocate.order(ByteOrder.nativeOrder());
            allocate.putInt(KerberosTime.getDefaultSkew());
            allocate.flip();
            byteChannel.write(allocate);
            return byteChannel;
        }
        
        private static void expunge(Path path, KerberosTime kerberosTime) throws IOException {
            Path tempFile = Files.createTempFile(path.getParent(), "rcache", null, new FileAttribute[0]);
            SeekableByteChannel byteChannel = Files.newByteChannel(path, new OpenOption[0]);
            Throwable t = null;
            try {
                SeekableByteChannel noClose = createNoClose(tempFile);
                Throwable t2 = null;
                Label_0281: {
                    try {
                        long n = (long)(kerberosTime.getSeconds() - readHeader(byteChannel));
                        while (true) {
                            try {
                                AuthTime from = AuthTime.readFrom(byteChannel);
                                if ((long)from.ctime > n) {
                                    noClose.write(ByteBuffer.wrap(from.encode(true)));
                                }
                                else {
                                    continue;
                                }
                                continue;
                            }
                            catch (BufferUnderflowException ex) {
                                if (noClose != null) {
                                    if (t2 != null) {
                                        try {
                                            noClose.close();
                                        }
                                        catch (Throwable exception) {
                                            t2.addSuppressed(exception);
                                        }
                                    }
                                    else {
                                        noClose.close();
                                    }
                                }
                                if (byteChannel != null) {
                                    if (t != null) {
                                        SeekableByteChannel seekableByteChannel = byteChannel;
                                        seekableByteChannel.close();
                                        break Label_0281;
                                    }
                                    byteChannel.close();
                                    break Label_0281;
                                }
                                else {
                                    break Label_0281;
                                }
                                Label_0182: {
                                    noClose.close();
                                }
                                Label_0189: {
                                    break Label_0189;
                                    iftrue(Label_0182:)(t2 == null);
                                }
                                noClose.close();
                            }
                        }
                    }
                    catch (Throwable t3) {}
                    finally {
                        Throwable t4;
                        throw t = t4;
                        Block_25: {
                            byteChannel.close();
                            Label_0278: {
                                byteChannel.close();
                                continue;
                            }
                            makeMine(tempFile);
                            Files.move(tempFile, path, new CopyOption[] { StandardCopyOption.REPLACE_EXISTING, StandardCopyOption.ATOMIC_MOVE });
                            return;
                            iftrue(Label_0278:)(t == null);
                            continue;
                        }
                    }
                }
                try {
                    SeekableByteChannel seekableByteChannel = byteChannel;
                    seekableByteChannel.close();
                }
                catch (Throwable t5) {}
            }
            catch (Throwable t4) {}
        }
        
        private int loadAndCheck(Path path, AuthTimeWithHash authTimeWithHash, KerberosTime kerberosTime) throws IOException, KrbApErrException {
            int n = 0;
            if (Files.isSymbolicLink(path)) {
                throw new IOException("Symlink not accepted");
            }
            try {
                Set<PosixFilePermission> posixFilePermissions = Files.getPosixFilePermissions(path, new LinkOption[0]);
                if (DflCache.uid != -1 && (int)Files.getAttribute(path, "unix:uid", new LinkOption[0]) != DflCache.uid) {
                    throw new IOException("Not mine");
                }
                if (posixFilePermissions.contains(PosixFilePermission.GROUP_READ) || posixFilePermissions.contains(PosixFilePermission.GROUP_WRITE) || posixFilePermissions.contains(PosixFilePermission.GROUP_EXECUTE) || posixFilePermissions.contains(PosixFilePermission.OTHERS_READ) || posixFilePermissions.contains(PosixFilePermission.OTHERS_WRITE) || posixFilePermissions.contains(PosixFilePermission.OTHERS_EXECUTE)) {
                    throw new IOException("Accessible by someone else");
                }
            }
            catch (UnsupportedOperationException ex) {}
            this.chan = Files.newByteChannel(path, new OpenOption[] { StandardOpenOption.WRITE, StandardOpenOption.READ });
            long n2 = (long)(kerberosTime.getSeconds() - readHeader(this.chan));
            long position = 0L;
            boolean n3 = false;
            while (true) {
                try {
                    position = this.chan.position();
                    AuthTime from = AuthTime.readFrom(this.chan);
                    if ((from instanceof AuthTimeWithHash)) {
                        if (authTimeWithHash.equals(from)) {
                            throw new KrbApErrException(34);
                        }
                        if (authTimeWithHash.isSameIgnoresHash(from)) {
                            n3 = true;
                        }
                    }
                    else if (authTimeWithHash.isSameIgnoresHash(from) && n3 == 0) {
                        throw new KrbApErrException(34);
                    }
                    if ((long)from.ctime < n2) {
                        ++n;
                    }
                    else {
                        --n;
                    }
                    continue;
                }
                catch (BufferUnderflowException ex2) {
                    this.chan.position(position);
                    return n;
                }
            }
        }
        
        private static int readHeader(SeekableByteChannel seekableByteChannel) throws IOException {
            ByteBuffer allocate = ByteBuffer.allocate(6);
            seekableByteChannel.read(allocate);
            if (allocate.getShort(0) != 1281) {
                throw new IOException("Not correct rcache version");
            }
            allocate.order(ByteOrder.nativeOrder());
            return allocate.getInt(2);
        }
        
        private void append(AuthTimeWithHash authTimeWithHash) throws IOException {
            this.chan.write(ByteBuffer.wrap(authTimeWithHash.encode(true)));
            this.chan.write(ByteBuffer.wrap(authTimeWithHash.encode(false)));
        }
        
        @Override
        public void close() throws IOException {
            if (this.chan != null) {
                this.chan.close();
            }
            this.chan = null;
        }
        
        static int access$100(DflCache.Storage storage, Path path, AuthTimeWithHash authTimeWithHash, KerberosTime kerberosTime) throws IOException, KrbApErrException {
            return storage.loadAndCheck(path, authTimeWithHash, kerberosTime);
        }
        
        static void access$200(Path path) throws IOException {
            create(path);
        }
        
        static void access$300(DflCache.Storage storage, AuthTimeWithHash authTimeWithHash) throws IOException {
            storage.append(authTimeWithHash);
        }
        
        static void access$400(Path path, KerberosTime kerberosTime) throws IOException {
            expunge(path, kerberosTime);
        }
    }
}
